@@ -6,15 +6,15 @@
 # @Filename: __init__.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
-# isort:skip_file
-
 import enum
 
+from typing import Type, Union
+
 
 class TypesEnumMeta(enum.EnumMeta):
     """Metaclass to allow initialising an Enum from a string."""
 
-    def __call__(cls, value):
+    def __call__(cls: Type[enum.Enum], value: Union[str, int]):  # type: ignore
 
         if isinstance(value, str):
             for flag in cls:
@@ -53,6 +53,8 @@ class CommandID(enum.IntEnum, metaclass=TypesEnumMeta):
     START_COGGING_CALIBRATION = 47
     SAVE_INTERNAL_CALIBRATION = 53
     GET_CURRENT = 56
+    SET_HOLDING_CURRENT = 112
+    GET_HOLDING_CURRENT = 113
     HALL_ON = 116
     HALL_OFF = 117
     ALPHA_CLOSED_LOOP_COLLISION_DETECTION = 118
@@ -63,6 +65,7 @@ class CommandID(enum.IntEnum, metaclass=TypesEnumMeta):
     BETA_CLOSED_LOOP_WITHOUT_COLLISION_DETECTION = 123
     BETA_OPEN_LOOP_COLLISION_DETECTION = 124
     BETA_OPEN_LOOP_WITHOUT_COLLISION_DETECTION = 125
+    GET_RAW_TEMPERATURE = 132
     START_FIRMWARE_UPGRADE = 200
     SEND_FIRMWARE_DATA = 201
 
@@ -73,35 +76,38 @@ class CommandID(enum.IntEnum, metaclass=TypesEnumMeta):
             return COMMAND_LIST[self]
 
 
-from .base import Command
-
 from .base import *
+from .base import Command
 from .bootloader import *
+from .calibration import *
 from .goto import *
 from .status import *
 from .trajectory import *
-from .calibration import *
 
 
 def all_subclasses(cls):
     """Recursive subclasses."""
 
-    return set(cls.__subclasses__()).union([s for c in cls.__subclasses__()
-                                            for s in all_subclasses(c)])
+    return set(cls.__subclasses__()).union(
+        [s for c in cls.__subclasses__() for s in all_subclasses(c)]
+    )
 
 
 # Generate a dictionary of commands
-COMMAND_LIST = {cclass.command_id: cclass
-                for cclass in all_subclasses(Command)
-                if cclass.command_id in CommandID}
+COMMAND_LIST = {
+    cclass.command_id: cclass
+    for cclass in all_subclasses(Command)
+    if cclass.command_id in CommandID
+}
 
 # Dynamically generate command classes for those commands for which we
 # didn't write a class. These classes are identical to a generic Command
 # but with a custom name.
 for cid in list(CommandID):
     if cid not in COMMAND_LIST:
-        CommandClass = type(cid.name.title().replace('_', ''),
-                            (Command,),
-                            {'command_id': cid,
-                             'broadcastable': False})
+        CommandClass = type(
+            cid.name.title().replace("_", ""),
+            (Command,),
+            {"command_id": cid, "broadcastable": False},
+        )
         COMMAND_LIST[cid] = CommandClass
