@@ -12,7 +12,7 @@ from jaeger.commands import Command, CommandID
 from jaeger.utils import bytes_to_int, int_to_bytes, motor_steps_to_angle
 
 
-__ALL__ = ['GetID', 'GetStatus', 'GetCurrent', 'GetActualPosition']
+__all__ = ["GetID", "GetStatus", "GetCurrent", "GetActualPosition"]
 
 
 class GetID(Command):
@@ -22,6 +22,7 @@ class GetID(Command):
     broadcastable = True
     timeout = 1
     safe = True
+    bootloader = True
 
     def get_ids(self):
         """Returns a list of positioners that replied back."""
@@ -35,6 +36,7 @@ class GetStatus(Command):
     command_id = CommandID.GET_STATUS
     broadcastable = True
     safe = True
+    bootloader = True
 
     def get_positioner_status(self):
         """Returns the positioner status flag for each reply."""
@@ -60,12 +62,12 @@ class GetActualPosition(Command):
         """
 
         if len(self.replies) == 0:
-            raise ValueError('no positioners have replied to this command.')
+            raise ValueError("no positioners have replied to this command.")
 
         data = self.replies[0].data
 
-        beta = bytes_to_int(data[4:], dtype='i4')
-        alpha = bytes_to_int(data[0:4], dtype='i4')
+        beta = bytes_to_int(data[4:], dtype="i4")
+        alpha = bytes_to_int(data[0:4], dtype="i4")
 
         return numpy.array(motor_steps_to_angle(alpha, beta))
 
@@ -75,7 +77,9 @@ class GetActualPosition(Command):
 
         alpha_motor, beta_motor = motor_steps_to_angle(alpha, beta, inverse=True)
 
-        data = int_to_bytes(int(alpha_motor), 'i4') + int_to_bytes(int(beta_motor), 'i4')
+        data = int_to_bytes(int(alpha_motor), "i4") + int_to_bytes(
+            int(beta_motor), "i4"
+        )
 
         return data
 
@@ -94,15 +98,39 @@ class GetCurrent(Command):
         ------
         ValueError
             If no reply has been received or the data cannot be parsed.
-
         """
 
         if len(self.replies) == 0:
-            raise ValueError('no positioners have replied to this command.')
+            raise ValueError("no positioners have replied to this command.")
 
         data = self.replies[0].data
 
-        beta = bytes_to_int(data[4:], dtype='i4')
-        alpha = bytes_to_int(data[0:4], dtype='i4')
+        beta = bytes_to_int(data[4:], dtype="i4")
+        alpha = bytes_to_int(data[0:4], dtype="i4")
 
         return numpy.array([alpha, beta])
+
+
+class GetTemperature(Command):
+    """Gets the temperature from the board temperature sensor, in C."""
+
+    command_id = CommandID.GET_RAW_TEMPERATURE
+    broadcastable = False
+    safe = True
+
+    def get_temperature(self) -> int:
+        """Returns the temperature in Celsius.
+
+        Raises
+        ------
+        ValueError
+            If no reply has been received or the data cannot be parsed.
+        """
+
+        if len(self.replies) == 0:
+            raise ValueError("no positioners have replied to this command.")
+
+        data = self.replies[0].data
+        rawT = bytes_to_int(data, dtype="u4")  # Raw temperature
+
+        return rawT
