@@ -12,7 +12,7 @@ from can import BusABC, Message
 
 
 class CANNetMessage(Message):
-    __slots__ = ('interface', 'bus')
+    __slots__ = ("interface", "bus")
 
 
 class CANNetBus(BusABC):
@@ -36,30 +36,38 @@ class CANNetBus(BusABC):
 
     # the supported bitrates and their commands
     _BITRATES = {
-        5000: '5',
-        10000: '10',
-        20000: '20',
-        50000: '50',
-        62500: '62.5',
-        83300: '83.3',
-        100000: '100',
-        125000: '125',
-        500000: '500',
-        800000: '800',
-        1000000: '1000'
+        5000: "5",
+        10000: "10",
+        20000: "20",
+        50000: "50",
+        62500: "62.5",
+        83300: "83.3",
+        100000: "100",
+        125000: "125",
+        500000: "500",
+        800000: "800",
+        1000000: "1000",
     }
 
     _REMOTE_PORT = 19228
 
-    LINE_TERMINATOR = b'\n'
+    LINE_TERMINATOR = b"\n"
 
-    def __init__(self, channel, port=None, bitrate=None, buses=[1], timeout=5, **kwargs):
+    def __init__(
+        self,
+        channel,
+        port=None,
+        bitrate=None,
+        buses=[1],
+        timeout=5,
+        **kwargs,
+    ):
 
         if not channel:  # if None or empty
-            raise TypeError('Must specify a TCP address.')
+            raise TypeError("Must specify a TCP address.")
 
         if not bitrate:
-            raise TypeError('Must specify a bitrate.')
+            raise TypeError("Must specify a bitrate.")
 
         port = port or self._REMOTE_PORT
 
@@ -77,7 +85,7 @@ class CANNetBus(BusABC):
 
         self.open()
 
-        self.channel_info = f'CAN@net channel={channel!r}, buses={self.buses!r}'
+        self.channel_info = f"CAN@net channel={channel!r}, buses={self.buses!r}"
 
         super(CANNetBus, self).__init__(channel, bitrate=None, **kwargs)
 
@@ -101,21 +109,26 @@ class CANNetBus(BusABC):
         self.close()
 
         if self.bitrate in self._BITRATES:
-            self._write_to_buses('CAN {bus} ' + f'INIT STD {self._BITRATES[self.bitrate]}')
-            self._write_to_buses('CAN {bus} ' + 'FILTER CLEAR')
-            self._write_to_buses('CAN {bus} ' + 'FILTER ADD EXT 00000000 00000000')
+            self._write_to_buses(
+                "CAN {bus} " + f"INIT STD {self._BITRATES[self.bitrate]}"
+            )
+            self._write_to_buses("CAN {bus} " + "FILTER CLEAR")
+            self._write_to_buses("CAN {bus} " + "FILTER ADD EXT 00000000 00000000")
         else:
-            raise ValueError('Invalid bitrate, choose one of ' +
-                             (', '.join(self._BITRATES)) + '.')
+            raise ValueError(
+                "Invalid bitrate, choose one of "
+                + (", ".join(map(str, self._BITRATES)))
+                + "."
+            )
 
-        self._write_to_buses('CAN {bus} START')
+        self._write_to_buses("CAN {bus} START")
 
         # Clear buffer
         self._socket.recv(8192)
 
     def close(self, buses=None):
 
-        self._write_to_buses('CAN {bus} STOP', buses=buses)
+        self._write_to_buses("CAN {bus} STOP", buses=buses)
 
     def _recv_internal(self, timeout):
 
@@ -128,7 +141,7 @@ class CANNetBus(BusABC):
         frame = []
 
         # Check that we don't have already a message
-        while (self.LINE_TERMINATOR not in self._buffer):
+        while self.LINE_TERMINATOR not in self._buffer:
             self._buffer += self._socket.recv(1)
 
         if self.LINE_TERMINATOR not in self._buffer:
@@ -144,12 +157,14 @@ class CANNetBus(BusABC):
         # Message is M 1 CSD 100 55 AA 55 AA or M 2 CED 18FE0201 01 02 03 04 05 06 07 08
         # Check if we have a message from the CAN network. Otherwise this is a message
         # from the device so we return it.
-        data = readStr.split(' ')
-        if data[0] != 'M':
-            msg = CANNetMessage(arbitration_id=0,
-                                timestamp=time.time(),
-                                dlc=0,
-                                data=msgStr)
+        data = readStr.split(" ")
+        if data[0] != "M":
+            msg = CANNetMessage(
+                arbitration_id=0,
+                timestamp=time.time(),
+                dlc=0,
+                data=msgStr,
+            )
             msg.interface = self
             msg.bus = None
             return msg, False
@@ -160,19 +175,19 @@ class CANNetBus(BusABC):
             return None, False
 
         # check if standard packet, FD not supported
-        if data[2][0] != 'C':
+        if data[2][0] != "C":
             return None, False
 
         # check if remote frame
-        if data[2][2] == 'D':
+        if data[2][2] == "D":
             remote = False
-        elif data[2][2] == 'R':
+        elif data[2][2] == "R":
             remote = True
 
         # check if standard or extended packet
-        if data[2][1] == 'S':
+        if data[2][1] == "S":
             extended = False
-        elif data[2][1] == 'E':
+        elif data[2][1] == "E":
             extended = True
         else:
             return None, False
@@ -187,12 +202,14 @@ class CANNetBus(BusABC):
             dlc = dlc + 1
 
         if canId is not None:
-            msg = CANNetMessage(arbitration_id=canId,
-                                is_extended_id=extended,
-                                timestamp=time.time(),
-                                is_remote_frame=remote,
-                                dlc=dlc,
-                                data=frame)
+            msg = CANNetMessage(
+                arbitration_id=canId,
+                is_extended_id=extended,
+                timestamp=time.time(),
+                is_remote_frame=remote,
+                dlc=dlc,
+                data=frame,
+            )
             msg.interface = self
             msg.bus = bus
             return msg, False
@@ -207,20 +224,20 @@ class CANNetBus(BusABC):
 
         for bus in buses:
 
-            sendStr = f'M {bus} '
+            sendStr = f"M {bus} "
 
             if msg.is_extended_id:
                 if msg.is_remote_frame:
-                    sendStr += f'CER {msg.arbitration_id:08X}'
+                    sendStr += f"CER {msg.arbitration_id:08X}"
                 else:
-                    sendStr += f'CED {msg.arbitration_id:08X}'
+                    sendStr += f"CED {msg.arbitration_id:08X}"
             else:
                 if msg.is_remote_frame:
-                    sendStr += f'CSR {msg.arbitration_id:03X}'
+                    sendStr += f"CSR {msg.arbitration_id:03X}"
                 else:
-                    sendStr += f'CSD {msg.arbitration_id:03X}'
+                    sendStr += f"CSD {msg.arbitration_id:03X}"
 
-            sendStr += ''.join([' %02X' % b for b in msg.data])
+            sendStr += "".join([" %02X" % b for b in msg.data])
 
             self.write(sendStr)
 
@@ -229,6 +246,6 @@ class CANNetBus(BusABC):
         self._socket.close()
 
     def fileno(self):
-        if hasattr(self._socket, 'fileno'):
+        if hasattr(self._socket, "fileno"):
             return self._socket.fileno()
         return -1
