@@ -6,13 +6,18 @@
 # @Filename: helpers.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
+from __future__ import annotations
+
 import asyncio
+import enum
 from concurrent.futures import Executor
 from contextlib import suppress
 from threading import Thread
 
+from typing import Callable, Generic, Optional, Type, TypeVar
 
-__ALL__ = ['AsyncQueue', 'StatusMixIn', 'PollerList', 'Poller', 'AsyncioExecutor']
+
+__all__ = ["AsyncQueue", "StatusMixIn", "PollerList", "Poller", "AsyncioExecutor"]
 
 
 class AsyncQueue(asyncio.Queue):
@@ -20,19 +25,17 @@ class AsyncQueue(asyncio.Queue):
 
     Parameters
     ----------
-    loop : event loop or `None`
-        The current event loop, or `asyncio.get_event_loop`.
-    callback : callable
+    callback
         A function to call when a new item is received from the queue. It can
         be a coroutine.
 
     """
 
-    def __init__(self, loop=None, callback=None):
-
-        async def process_queue(loop):
+    def __init__(self, callback: Optional[Callable] = None):
+        async def process_queue():
             """Waits for the next item and sends it to the cb function."""
 
+            loop = asyncio.get_running_loop()
             while True:
                 item = await self.get()
                 if callback:
@@ -40,12 +43,13 @@ class AsyncQueue(asyncio.Queue):
 
         super().__init__()
 
-        loop = loop or asyncio.get_event_loop()
-
-        self.watcher = loop.create_task(process_queue(loop))
+        self.watcher = asyncio.create_task(process_queue())
 
 
-class StatusMixIn(object):
+Status_co = TypeVar("Status_co", bound=enum.Enum)
+
+
+class StatusMixIn(Generic[Status_co]):
     """A mixin that provides status tracking with callbacks.
 
     Provides a status property that executes a list of callbacks when
@@ -53,25 +57,30 @@ class StatusMixIn(object):
 
     Parameters
     ----------
-    maskbit_flags : class
+    maskbit_flags
         A class containing the available statuses as a series of maskbit
         flags. Usually as subclass of `enum.Flag`.
-    initial_status : str
+    initial_status
         The initial status.
-    callback_func : function
+    callback_func
         The function to call if the status changes.
-    call_now : bool
+    call_now
         Whether the callback function should be called when initialising.
 
     Attributes
     ----------
-    callbacks : list
+    callbacks
         A list of the callback functions to call.
 
     """
 
-    def __init__(self, maskbit_flags, initial_status=None,
-                 callback_func=None, call_now=False):
+    def __init__(
+        self,
+        maskbit_flags: Type[Status_co],
+        initial_status: Optional[Status_co] = None,
+        callback_func: Optional[Callable] = None,
+        call_now: bool = False,
+    ):
 
         self._flags = maskbit_flags
         self.callbacks = []
@@ -84,12 +93,12 @@ class StatusMixIn(object):
         if call_now is True:
             self.do_callbacks()
 
-    def add_callback(self, cb):
+    def add_callback(self, cb: Callable):
         """Adds a callback."""
 
         self.callbacks.append(cb)
 
-    def remove_callback(self, cb):
+    def remove_callback(self, cb: Callable):
         """Removes a callback."""
 
         self.callbacks.remove(cb)
@@ -97,20 +106,21 @@ class StatusMixIn(object):
     def do_callbacks(self):
         """Calls functions in ``callbacks``."""
 
-        assert hasattr(self, 'callbacks'), \
-            'missing callbacks attribute. Did you call __init__()?'
+        assert hasattr(
+            self, "callbacks"
+        ), "missing callbacks attribute. Did you call __init__()?"
 
         for func in self.callbacks:
             func()
 
     @property
-    def status(self):
+    def status(self) -> Status_co | None:
         """Returns the status."""
 
         return self._status
 
     @status.setter
-    def status(self, value):
+    def status(self, value: Status_co):
         """Sets the status."""
 
         if value != self._status:
@@ -132,19 +142,16 @@ class StatusMixIn(object):
         self._flags = value
         self._status = None
 
-    async def wait_for_status(self, value, loop=None):
+    async def wait_for_status(
+        self,
+        value,
+    ):
         """Awaits until the status matches ``value``."""
 
         if self.status == value:
             return
 
-        if loop is None:
-            if hasattr(self, 'loop') and self.loop is not None:
-                loop = self.loop
-            else:
-                loop = asyncio.get_event_loop()
-
-        self.watcher = asyncio.Event(loop=loop)
+        self.watcher = asyncio.Event()
 
         while self.status != value:
             await self.watcher.wait()
@@ -160,7 +167,7 @@ class PollerList(list):
     def __init__(self, pollers=[]):
 
         names = [poller.name for poller in pollers]
-        assert len(names) == len(set(names)), 'repeated names in poller list.'
+        assert len(names) == len(set(names)), "repeated names in poller list."
 
         list.__init__(self, pollers)
 
@@ -173,12 +180,13 @@ class PollerList(list):
     def append(self, poller):
         """Adds a poller."""
 
-        assert isinstance(poller, Poller), 'not a poller.'
+        assert isinstance(poller, Poller), "not a poller."
 
         names = [pp.name for pp in self]
         if poller.name in names:
-            raise ValueError(f'a poller with name {poller.name} is '
-                             'already in the list.')
+            raise ValueError(
+                f"a poller with name {poller.name} is already in the list."
+            )
 
         list.append(self, poller)
 
@@ -213,8 +221,9 @@ class PollerList(list):
 
         """
 
-        delay_coros = [poller.set_delay(delay=delay, immediate=immediate)
-                       for poller in self]
+        delay_coros = [
+            poller.set_delay(delay=delay, immediate=immediate) for poller in self
+        ]
         await asyncio.gather(*delay_coros)
 
     def start(self, delay=None):
@@ -268,7 +277,7 @@ class Poller(object):
         self._orig_delay = delay
         self.delay = delay
 
-        self.loop = loop or asyncio.get_event_loop()
+        self.loop: asyncio.AbstractEventLoop = loop or asyncio.get_event_loop()
 
         # Create two tasks, one for the sleep timer and another for the poller
         # itself. We do this because we want to be able to cancell the sleep
@@ -290,8 +299,9 @@ class Poller(object):
                 if ee.__class__ == asyncio.CancelledError:
                     raise
                 if not self._task.cancelled:
-                    self.loop.call_exception_handler({'message': 'failed running callback',
-                                                      'exception': ee})
+                    self.loop.call_exception_handler(
+                        {"message": "failed running callback", "exception": ee}
+                    )
 
             self._sleep_task = self.loop.create_task(asyncio.sleep(self.delay))
 
@@ -353,7 +363,8 @@ class Poller(object):
         self._task.cancel()
 
         with suppress(asyncio.CancelledError):
-            await self._task
+            if self._task is not None:
+                await self._task
 
     async def call_now(self):
         """Calls the callback immediately."""
